@@ -549,8 +549,8 @@ public class PDFParser extends BaseParser
         else
         {
             long number = -1;
-            int genNum = -1;
-            String objectKey = null;
+            int genNum;
+            String objectKey;
             boolean missingObjectNumber = false;
             try
             {
@@ -752,10 +752,6 @@ public class PDFParser extends BaseParser
         {
             long currObjID = readObjectNumber(); // first obj id
             long count = readLong(); // the number of objects in the xref table
-            if (count == 0)
-            {
-                LOG.warn("Count in xref table is 0 at offset " + pdfSource.getOffset());
-            }
             skipSpaces();
             for(int i = 0; i < count; i++)
             {
@@ -769,7 +765,7 @@ public class PDFParser extends BaseParser
                 }
                 //Ignore table contents
                 String currentLine = readLine();
-                String[] splitString = currentLine.split(" ");
+                String[] splitString = currentLine.split("\\s");
                 if (splitString.length < 3)
                 {
                     LOG.warn("invalid xref line: " + currentLine);
@@ -888,8 +884,26 @@ public class PDFParser extends BaseParser
      */
     public void parseXrefStream( COSStream stream, long objByteOffset ) throws IOException
     {
-        xrefTrailerResolver.nextXrefObj( objByteOffset );
-        xrefTrailerResolver.setTrailer( stream );
+        parseXrefStream(stream, objByteOffset, true);
+    }
+
+    /**
+     * Fills XRefTrailerResolver with data of given stream.
+     * Stream must be of type XRef.
+     * @param stream the stream to be read
+     * @param objByteOffset the offset to start at
+     * @param isStandalone should be set to true if the stream is not part of a hybrid xref table
+     * @throws IOException if there is an error parsing the stream
+     */
+    public void parseXrefStream( COSStream stream, long objByteOffset, boolean isStandalone ) throws IOException
+    {
+        // the cross reference stream of a hybrid xref table will be added to the existing one
+        // and we must not override the offset and the trailer
+        if (isStandalone)
+        {
+            xrefTrailerResolver.nextXrefObj( objByteOffset );
+            xrefTrailerResolver.setTrailer( stream );
+        }
         PDFXrefStreamParser parser =
             new PDFXrefStreamParser( stream, document, forceParsing, xrefTrailerResolver );
         parser.parse();
@@ -907,16 +921,17 @@ public class PDFParser extends BaseParser
     private static class ConflictObj
     {
 
-        private long offset;
-        private COSObjectKey objectKey;
-        private COSObject object;
+        private final long offset;
+        private final COSObjectKey objectKey;
+        private final COSObject object;
 
-        public ConflictObj(long offsetValue, COSObjectKey key, COSObject pdfObject)
+        ConflictObj(long offsetValue, COSObjectKey key, COSObject pdfObject)
         {
             this.offset = offsetValue;
             this.objectKey = key;
             this.object = pdfObject;
         }
+        
         @Override
         public String toString()
         {
@@ -940,8 +955,7 @@ public class PDFParser extends BaseParser
                 do
                 {
                     ConflictObj o = conflicts.next();
-                    Long offset = new Long(o.offset);
-                    if (tolerantConflicResolver(values, offset, 4))
+                    if (tolerantConflicResolver(values, o.offset, 4))
                     {
                         COSObject pdfObject = document.getObjectFromPool(o.objectKey);
                         if (pdfObject.getObjectNumber() != null 
@@ -951,7 +965,7 @@ public class PDFParser extends BaseParser
                         }
                         else
                         {
-                            LOG.debug("Conflict object ["+o.objectKey+"] at offset "+offset
+                            LOG.debug("Conflict object [" + o.objectKey + "] at offset " + o.offset
                                     +" found in the xref table, but the object numbers differ. Ignoring this object."
                                     + " The document is maybe malformed.");
                         }
