@@ -48,7 +48,7 @@ public class PDFStreamParser extends BaseParser
 {
     private List<Object> streamObjects = new ArrayList<Object>( 100 );
     private final RandomAccess file;
-    private final int    maxBinCharTestLength = 5;
+    private final int    maxBinCharTestLength = 10;
     private final byte[] binCharTestArr = new byte[maxBinCharTestLength];
 
     /**
@@ -132,7 +132,7 @@ public class PDFStreamParser extends BaseParser
     {
         try
         {
-            Object token = null;
+            Object token;
             while( (token = parseNextToken()) != null )
             {
                 streamObjects.add( token );
@@ -229,7 +229,7 @@ public class PDFStreamParser extends BaseParser
      */
     private Object parseNextToken() throws IOException
     {
-        Object retval = null;
+        Object retval;
 
         skipSpaces();
         int nextByte = pdfSource.peek();
@@ -247,7 +247,6 @@ public class PDFStreamParser extends BaseParser
                 pdfSource.unread( leftBracket ); //put back first bracket
                 if(c == '<')
                 {
-
                     COSDictionary pod = parseCOSDictionary();
                     skipSpaces();
                     if((char)pdfSource.peek() == 's')
@@ -395,11 +394,12 @@ public class PDFStreamParser extends BaseParser
                 // PDF spec is kinda unclear about this. Should a whitespace
                 // always appear before EI? Not sure, so that we just read
                 // until EI<whitespace>.
-                // Be aware not all kind of whitespaces are allowed here. see PDFBOX1561
+                // Be aware not all kind of whitespaces are allowed here. see PDFBOX-1561
                 while( !(lastByte == 'E' &&
                          currentByte == 'I' &&
-                         isSpaceOrReturn() &&
-                         hasNoFollowingBinData( pdfSource )) &&
+                         hasNextSpaceOrReturn() &&
+                         hasNoFollowingBinData( pdfSource ) &&
+                         !hasPrecedingAscii85Data(imageData)) &&
                        !pdfSource.isEOF() )
                 {
                     imageData.write( lastByte );
@@ -408,7 +408,7 @@ public class PDFStreamParser extends BaseParser
                 }
                 // the EI operator isn't unread, as it won't be processed anyway
                 retval = PDFOperator.getOperator( "ID" );
-                // save the image data to the operator, so that it can be accessed it later
+                // save the image data to the operator, so that it can be accessed later
                 ((PDFOperator)retval).setImageData( imageData.toByteArray() );
                 break;
             }
@@ -434,18 +434,17 @@ public class PDFStreamParser extends BaseParser
                     retval = PDFOperator.getOperator( operator );
                 }
             }
-
         }
-
         return retval;
     }
 
     /**
-     * Looks up next 5 bytes if they contain only ASCII characters (no control
-     * sequences etc.).
+     * Looks up an amount of bytes if they contain only ASCII characters (no
+     * control sequences etc.), and that these ASCII characters begin with a
+     * sequence of 1-3 non-blank characters between blanks
      *
-     * @return <code>true</code> if next 5 bytes are printable ASCII characters,
-     * otherwise <code>false</code>
+     * @return <code>true</code> if next bytes are probably printable ASCII
+     * characters starting with a PDF operator, otherwise <code>false</code>
      */
     private boolean hasNoFollowingBinData(final PushbackInputStream pdfSource) 
             throws IOException
@@ -453,6 +452,8 @@ public class PDFStreamParser extends BaseParser
         // as suggested in PDFBOX-1164
         final int readBytes = pdfSource.read(binCharTestArr, 0, maxBinCharTestLength);
         boolean noBinData = true;
+        int startOpIdx = -1;
+        int endOpIdx = -1;
         
         if (readBytes > 0)
         {
@@ -465,12 +466,60 @@ public class PDFStreamParser extends BaseParser
                     noBinData = false;
                     break;
                 }
+                // find the start of a PDF operator
+                if (startOpIdx == -1 && (b == 9 || b == 0x20 || b == 0x0a || b == 0x0d))
+                {
+                    startOpIdx = bIdx;
+                }
+                else if (startOpIdx != -1 && endOpIdx == -1 && (b == 9 || b == 0x20 || b == 0x0a || b == 0x0d))
+                {
+                    if (bIdx == startOpIdx + 1)
+                    {
+                        // several blanks after another
+                        startOpIdx = bIdx;
+                    }
+                    else
+                    {
+                        endOpIdx = bIdx;
+                    }
+                }
+            }
+            // a PDF operator is 1-3 bytes long
+            if (endOpIdx == -1 || startOpIdx == -1 || endOpIdx - startOpIdx > 3)
+            {
+                noBinData = false;
             }
             pdfSource.unread(binCharTestArr, 0, readBytes);
         }
         return noBinData;
     }
 
+    /**
+     * Check whether the output stream ends with 70 ASCII85 data bytes
+     * (33..117). This method is to be called when "EI" and then space/LF/CR
+     * are detected.
+     *
+     * @param imageData output data stream without the "EI"
+     * @return true if this is an ASCII85 line so the "EI" is to be considered
+     * part of the data stream, false if not
+     */
+    private boolean hasPrecedingAscii85Data(ByteArrayOutputStream imageData)
+    {
+        if (imageData.size() < 70)
+        {
+            return false;
+        }
+        byte[] tab = imageData.toByteArray();
+        for (int i = tab.length - 1; i >= tab.length - 70; --i)
+        {
+            if (tab[i] < 33 || tab[i] > 117)
+            {
+                return false;
+            }
+        }
+        return true;
+    }
+    
     /**
      * This will read an operator from the stream.
      *
@@ -510,6 +559,7 @@ public class PDFStreamParser extends BaseParser
         return buffer.toString();
     }
     
+    
     private boolean isSpaceOrReturn( int c )
     {
         return c == 10 || c == 13 || c == 32;
@@ -521,7 +571,7 @@ public class PDFStreamParser extends BaseParser
      * @return true if the next char is a space or a return
      * @throws IOException if something went wrong
      */
-    private boolean isSpaceOrReturn() throws IOException
+    private boolean hasNextSpaceOrReturn() throws IOException
     {
         return isSpaceOrReturn( pdfSource.peek() );
     }
