@@ -18,6 +18,7 @@ package org.apache.pdfbox.pdfwriter;
 
 import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
+import java.io.Closeable;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
@@ -73,7 +74,7 @@ import org.apache.pdfbox.util.StringUtil;
  * @author <a href="mailto:ben@benlitchfield.com">Ben Litchfield</a>
  * 
  */
-public class COSWriter implements ICOSVisitor
+public class COSWriter implements ICOSVisitor, Closeable
 {
     /**
      * The dictionary open token.
@@ -504,10 +505,14 @@ public class COSWriter implements ICOSVisitor
             if(obj instanceof COSDictionary) 
             {
                 COSDictionary dict = (COSDictionary)obj;
-                COSName item = (COSName)dict.getItem(COSName.TYPE);
-                if (COSName.SIG.equals(item) || COSName.DOC_TIME_STAMP.equals(item))
+                COSBase itemType = dict.getItem(COSName.TYPE);
+                if (itemType instanceof COSName)
                 {
-                    reachedSignature = true;
+                    COSName item = (COSName) itemType;
+                    if (COSName.SIG.equals(item) || COSName.DOC_TIME_STAMP.equals(item))
+                    {
+                        reachedSignature = true;
+                    }
                 }
             }
             
@@ -778,12 +783,13 @@ public class COSWriter implements ICOSVisitor
             SignatureInterface signatureInterface = doc.getSignatureInterface();
             byte[] sign = signatureInterface.sign(new ByteArrayInputStream(pdfContent));
             String signature = new COSString(sign).getHexString();
-            int leftSignaturerange = signaturePosition[1]-signaturePosition[0]-signature.length();
-            if(leftSignaturerange<0)
+            int startPos = signaturePosition[0] + 1; // move past "<"
+            int endPos = signaturePosition[1] - 1; // move in front of ">"
+            if (startPos + signature.length() > endPos)            
             {
                 throw new IOException("Can't write signature, not enough space");
             }
-            getStandardOutput().setPos(signaturePosition[0]+1);
+            getStandardOutput().setPos(startPos);
             getStandardOutput().write(signature.getBytes());
         }
     }
@@ -918,15 +924,15 @@ public class COSWriter implements ICOSVisitor
                 COSBase current = i.next();
                 if( current instanceof COSDictionary )
                 {
-                	if (current.isDirect())
-                	{
-                		visitFromDictionary((COSDictionary)current);
-                	}
-                	else
-                	{
-                		addObjectToWrite( current );
-                		writeReference( current );
-                	}
+                    if (current.isDirect())
+                    {
+                        visitFromDictionary((COSDictionary)current);
+                    }
+                    else
+                    {
+                        addObjectToWrite( current );
+                        writeReference( current );
+                    }
                 }
                 else if( current instanceof COSObject )
                 {
@@ -947,7 +953,7 @@ public class COSWriter implements ICOSVisitor
                 }
                 else if( current instanceof COSString )
                 {
-                    COSString copy = new COSString(true);
+                    COSString copy = new COSString();
                     copy.append(((COSString)current).getBytes());
                     copy.accept(this);
                 }
@@ -1427,7 +1433,7 @@ public class COSWriter implements ICOSVisitor
      * @throws COSVisitorException If an error occurs while generating the data.
      */
     public void write(PDDocument doc) throws COSVisitorException
-	{
+    {
         Long idTime = doc.getDocumentId() == null ? System.currentTimeMillis() : 
                                                     doc.getDocumentId();
         
