@@ -21,7 +21,7 @@ import java.awt.print.Pageable;
 import java.awt.print.Printable;
 import java.awt.print.PrinterException;
 import java.awt.print.PrinterJob;
-import java.io.BufferedInputStream;
+import java.io.Closeable;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
@@ -45,7 +45,6 @@ import org.apache.pdfbox.cos.COSObject;
 import org.apache.pdfbox.cos.COSStream;
 import org.apache.pdfbox.exceptions.COSVisitorException;
 import org.apache.pdfbox.exceptions.CryptographyException;
-import org.apache.pdfbox.exceptions.InvalidPasswordException;
 import org.apache.pdfbox.exceptions.SignatureException;
 import org.apache.pdfbox.io.RandomAccess;
 import org.apache.pdfbox.pdfparser.BaseParser;
@@ -85,7 +84,7 @@ import org.apache.pdfbox.pdmodel.interactive.form.PDSignatureField;
  * @author <a href="mailto:ben@benlitchfield.com">Ben Litchfield</a>
  * @version $Revision: 1.47 $
  */
-public class PDDocument implements Pageable
+public class PDDocument implements Pageable, Closeable
 {
 
     private COSDocument document;
@@ -130,10 +129,8 @@ public class PDDocument implements Pageable
     /**
      * Constructor, creates a new PDF Document with no pages.  You need to add
      * at least one page for the document to be valid.
-     *
-     * @throws IOException If there is an error creating this document.
      */
-    public PDDocument() throws IOException
+    public PDDocument()
     {
         document = new COSDocument();
 
@@ -308,11 +305,11 @@ public class PDDocument implements Pageable
         int preferedSignatureSize = options.getPreferedSignatureSize();
         if (preferedSignatureSize > 0)
         {
-            sigObject.setContents(new byte[preferedSignatureSize * 2 + 2]);
+            sigObject.setContents(new byte[preferedSignatureSize]);
         }
         else
         {
-            sigObject.setContents(new byte[0x2500 * 2 + 2]);
+            sigObject.setContents(new byte[0x2500]);
         }
         
         // Reserve ByteRange
@@ -935,16 +932,14 @@ public class PDDocument implements Pageable
      *
      * @throws CryptographyException If there is an error decrypting the document.
      * @throws IOException If there is an error getting the stream data.
-     * @throws InvalidPasswordException If the password is not a user or owner password.
      *
      */
-    public void decrypt( String password ) throws CryptographyException, IOException, InvalidPasswordException
+    public void decrypt( String password ) throws CryptographyException, IOException
     {
         try
         {
             StandardDecryptionMaterial m = new StandardDecryptionMaterial(password);
             this.openProtection(m);
-            document.dereferenceObjectStreams();
         }
         catch(BadSecurityHandlerException e)
         {
@@ -1215,7 +1210,7 @@ public class PDDocument implements Pageable
      */
     public static PDDocument load( InputStream input, RandomAccess scratchFile ) throws IOException
     {
-        PDFParser parser = new PDFParser( new BufferedInputStream( input ) , scratchFile );
+        PDFParser parser = new PDFParser(input, scratchFile);
         parser.parse();
         return parser.getPDDocument();
     }
@@ -1234,7 +1229,7 @@ public class PDDocument implements Pageable
      */
     public static PDDocument load(InputStream input, RandomAccess scratchFile, boolean force) throws IOException
     {
-        PDFParser parser = new PDFParser( new BufferedInputStream( input ), scratchFile, force);
+        PDFParser parser = new PDFParser(input, scratchFile, force);
         parser.parse();
         return parser.getPDDocument();
     }
@@ -1600,6 +1595,7 @@ public class PDDocument implements Pageable
             securityHandler.decryptDocument(this, pm);
             document.dereferenceObjectStreams();
             document.setEncryptionDictionary( null );
+            getDocumentCatalog();
         }
         else
         {
