@@ -35,6 +35,8 @@ import java.util.Iterator;
 import javax.crypto.Cipher;
 import javax.crypto.KeyGenerator;
 import javax.crypto.SecretKey;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 
 import org.bouncycastle.asn1.ASN1InputStream;
 import org.bouncycastle.asn1.DERObject;
@@ -65,7 +67,7 @@ import org.apache.pdfbox.pdmodel.PDDocument;
  * This class implements the public key security handler
  * described in the PDF specification.
  *
- * @see PDF Spec 1.6 p104
+ * [PDF Spec 1.6 p104]
  *
  * @see PublicKeyProtectionPolicy to see how to protect document with this security handler.
  *
@@ -74,6 +76,11 @@ import org.apache.pdfbox.pdmodel.PDDocument;
  */
 public class PublicKeySecurityHandler extends SecurityHandler
 {
+        
+    /**
+     * Log instance.
+     */
+    private static final Log LOG = LogFactory.getLog(PublicKeySecurityHandler.class);
 
     /**
      * The filter name.
@@ -131,7 +138,7 @@ public class PublicKeySecurityHandler extends SecurityHandler
      * called from there. Only if decryption of single objects is needed this should be called instead.
      *
      * @param encDictionary  encryption dictionary, can be retrieved via {@link PDDocument#getEncryptionDictionary()}
-     * @param documentIDArray  document id which is returned via {@link COSDocument#getDocumentID()} (not used by this handler)
+     * @param documentIDArray  document id which is returned via {@link org.apache.pdfbox.cos.COSDocument#getDocumentID()} (not used by this handler)
      * @param decryptionMaterial Information used to decrypt the document.
      *
      * @throws IOException If there is an error accessing data.
@@ -141,18 +148,19 @@ public class PublicKeySecurityHandler extends SecurityHandler
 				 														 DecryptionMaterial decryptionMaterial)
     throws CryptographyException, IOException
     {
-    	
-	      if(encDictionary.getLength() != 0)
-	      {
-	          this.keyLength = encDictionary.getLength();
-	      }
-	
 	      if(!(decryptionMaterial instanceof PublicKeyDecryptionMaterial))
 	      {
 	          throw new CryptographyException(
 	              "Provided decryption material is not compatible with the document");
 	      }
-	
+
+	      decryptMetadata = encDictionary.isEncryptMetaData();
+          if(encDictionary.getLength() != 0)
+          {
+              this.keyLength = encDictionary.getLength();
+          }
+    
+	      
 	      PublicKeyDecryptionMaterial material = (PublicKeyDecryptionMaterial)decryptionMaterial;
 	
 	      try
@@ -370,7 +378,7 @@ public class PublicKeySecurityHandler extends SecurityHandler
         }
         catch(Exception e)
         {
-            e.printStackTrace();
+            LOG.error(e,e);
             throw new CryptographyException(e);
         }
 
