@@ -25,6 +25,8 @@ import java.util.List;
 import java.util.Map;
 
 import org.apache.fontbox.afm.AFMParser;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 import org.apache.fontbox.afm.FontMetric;
 import org.apache.fontbox.cmap.CMapParser;
 import org.apache.fontbox.cmap.CMap;
@@ -58,6 +60,11 @@ import java.awt.geom.AffineTransform;
 public abstract class PDFont implements COSObjectable
 {
 
+    /**
+     * Log instance.
+     */
+    private static final Log LOG = LogFactory.getLog(PDFont.class);
+
     /**
      * The cos dictionary for this font.
      */
@@ -90,6 +97,8 @@ public abstract class PDFont implements COSObjectable
     protected CMap toUnicodeCmap = null;
     
     private boolean hasToUnicode = false;
+    
+    private boolean widthsAreMissing = false;
 
     protected static Map<String, CMap> cmapObjects =
         Collections.synchronizedMap( new HashMap<String, CMap>() );
@@ -124,6 +133,13 @@ public abstract class PDFont implements COSObjectable
         addAdobeFontMetric( metrics, "Times-Italic" );
         addAdobeFontMetric( metrics, "Times-Roman" );
         addAdobeFontMetric( metrics, "ZapfDingbats" );
+
+        // PDFBOX-239
+        addAdobeFontMetric(metrics, "Arial", "Helvetica");
+        addAdobeFontMetric(metrics, "Arial,Bold", "Helvetica-Bold");
+        addAdobeFontMetric(metrics, "Arial,Italic", "Helvetica-Oblique");
+        addAdobeFontMetric(metrics, "Arial,BoldItalic", "Helvetica-BoldOblique");
+        
         return metrics;
     }
 
@@ -132,10 +148,15 @@ public abstract class PDFont implements COSObjectable
 
     private static void addAdobeFontMetric(
             Map<String, FontMetric> metrics, String name )
+    {
+        addAdobeFontMetric(metrics, name, name);
+    }
+    
+    private static void addAdobeFontMetric(Map<String, FontMetric> metrics, String name, String filePrefix)
     {
         try
         {
-            String resource = resourceRootAFM + name + ".afm";
+            String resource = resourceRootAFM + filePrefix + ".afm";
             InputStream afmStream = ResourceLoader.loadResource( resource );
             if( afmStream != null )
             {
@@ -265,13 +286,13 @@ public abstract class PDFont implements COSObjectable
     public abstract float getFontWidth( byte[] c, int offset, int length ) throws IOException;
 
     /**
-     * This will get the font width for a character.
+     * This will get the font height for a character.
      *
-     * @param c The character code to get the width for.
+     * @param c The character code to get the height for.
      * @param offset The offset into the array.
      * @param length The length of the data.
      *
-     * @return The width is in 1000 unit of text space, ie 333 or 777
+     * @return The height is in 1000 unit of text space, ie 333 or 777
      *
      * @throws IOException If an error occurs while parsing.
      */
@@ -556,7 +577,7 @@ public abstract class PDFont implements COSObjectable
             catch (UnsupportedEncodingException e)
             {
                 // Nothing should happen here
-                e.printStackTrace();
+                LOG.error(e,e);
             }
             for( int j=0; j<256; j++ )
             {
@@ -567,7 +588,7 @@ public abstract class PDFont implements COSObjectable
                 catch (UnsupportedEncodingException e)
                 {
                     // Nothing should happen here
-                    e.printStackTrace();
+                    LOG.error(e,e);
                 }
             }
         }
@@ -606,7 +627,10 @@ public abstract class PDFont implements COSObjectable
                     cmapObjects.put( targetCmap.getName(), targetCmap );
                 }
             }
-            catch (IOException exception) {}
+            catch (IOException exception)
+            {
+                LOG.error("An error occurs while reading a CMap", exception);
+            }
         }
         return targetCmap;
     }
@@ -785,13 +809,17 @@ public abstract class PDFont implements COSObjectable
      */
     public List<Float> getWidths()
     {
-        if (widths == null)
+        if (widths == null && !widthsAreMissing)
         {
             COSArray array = (COSArray)font.getDictionaryObject( COSName.WIDTHS );
             if (array != null)
             {
                 widths = COSArrayList.convertFloatCOSArrayToList(array);
             }
+            else
+            {
+                widthsAreMissing = true;
+            }
         }
         return widths;
     }
@@ -875,10 +903,13 @@ public abstract class PDFont implements COSObjectable
         if (charCode >= firstChar && charCode <= lastChar)
         {
             // maybe the font doesn't provide any widths
-            getWidths();
-            if (widths != null)
+            if (!widthsAreMissing)
             {
-                width = widths.get(charCode-firstChar).floatValue();
+                getWidths();
+                if (widths != null)
+                {
+                    width = widths.get(charCode-firstChar).floatValue();
+                }
             }
         }
         else
@@ -925,4 +956,12 @@ public abstract class PDFont implements COSObjectable
     {
         return toUnicodeCmap;
     }
+    
+    /**
+     * Calling this will release all cached information.
+     */
+    public void clear()
+    {
+    }
+
 }
