@@ -17,12 +17,9 @@
 package org.apache.pdfbox.pdmodel.graphics.shading;
 
 import java.awt.PaintContext;
-import java.awt.Transparency;
-import java.awt.color.ColorSpace;
+import java.awt.Rectangle;
 import java.awt.geom.AffineTransform;
 import java.awt.image.ColorModel;
-import java.awt.image.ComponentColorModel;
-import java.awt.image.DataBuffer;
 import java.awt.image.Raster;
 import java.awt.image.WritableRaster;
 import java.io.IOException;
@@ -32,116 +29,91 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBoolean;
 import org.apache.pdfbox.pdmodel.common.function.PDFunction;
-import org.apache.pdfbox.pdmodel.graphics.color.PDColorSpace;
-import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceN;
-import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceRGB;
-import org.apache.pdfbox.pdmodel.graphics.color.PDSeparation;
 import org.apache.pdfbox.util.Matrix;
 
 /**
- * This class represents the PaintContext of an axial shading.
- * 
- * @author lehmi
- * 
+ * AWT PaintContext for axial shading.
+ *
+ * Performance improvement done as part of GSoC2014, Tilman Hausherr is the
+ * mentor.
+ *
+ * @author Andreas Lehmkhler
+ * @author Shaola Ren
+ *
  */
-public class AxialShadingContext implements PaintContext 
+public class AxialShadingContext extends ShadingContext implements PaintContext
 {
-
-    private ColorModel outputColorModel;
-    private ColorSpace shadingColorSpace;
-    private PDFunction shadingTinttransform;
-    private PDShadingType2 shadingType;
-
-    private float[] coords;
-    private float[] domain;
-    private float[] background;
-    private boolean[] extend;
-    private double x1x0; 
-    private double y1y0;
-    private float d1d0;
-    private double denom;
-    
-    /**
-     * Log instance.
-     */
     private static final Log LOG = LogFactory.getLog(AxialShadingContext.class);
 
+    private PDShadingType2 axialShadingType;
+
+    private final float[] coords;
+    private final float[] domain;
+    private float[] background;
+    private int rgbBackground;
+    private final boolean[] extend;
+    private final double x1x0;
+    private final double y1y0;
+    private final float d1d0;
+    private double denom;
+
+    private final double axialLength;
+    private final int[] colorTable;
+
     /**
      * Constructor creates an instance to be used for fill operations.
-     * 
-     * @param shadingType2 the shading type to be used
-     * @param colorModelValue the color model to be used
+     *
+     * @param shading the shading type to be used
+     * @param colorModel the color model to be used
      * @param xform transformation for user to device space
-     * @param ctm current transformation matrix
+     * @param ctm the transformation matrix
      * @param pageHeight height of the current page
-     * 
+     * @param dBounds device bounds
      */
-    public AxialShadingContext(PDShadingType2 shadingType2, ColorModel colorModelValue, 
-            AffineTransform xform, Matrix ctm, int pageHeight) 
+    public AxialShadingContext(PDShadingType2 shading, ColorModel colorModel, AffineTransform xform,
+            Matrix ctm, int pageHeight, Rectangle dBounds) throws IOException
     {
-        shadingType = shadingType2;
-        coords = shadingType2.getCoords().toFloatArray();
+        super(shading, colorModel, xform, ctm, pageHeight, dBounds);
+        this.axialShadingType = shading;
+        coords = shading.getCoords().toFloatArray();
+
         if (ctm != null)
         {
             // the shading is used in combination with the sh-operator
-            float[] coordsTemp = new float[coords.length]; 
             // transform the coords from shading to user space
-            ctm.createAffineTransform().transform(coords, 0, coordsTemp, 0, 2);
+            ctm.createAffineTransform().transform(coords, 0, coords, 0, 2);
             // move the 0,0-reference
-            coordsTemp[1] = pageHeight - coordsTemp[1];
-            coordsTemp[3] = pageHeight - coordsTemp[3];
-            // transform the coords from user to device space
-            xform.transform(coordsTemp, 0, coords, 0, 2);
+            coords[1] = pageHeight - coords[1];
+            coords[3] = pageHeight - coords[3];
         }
         else
         {
             // the shading is used as pattern colorspace in combination
             // with a fill-, stroke- or showText-operator
-            float translateY = (float)xform.getTranslateY();
+            float translateY = (float) xform.getTranslateY();
             // move the 0,0-reference including the y-translation from user to device space
             coords[1] = pageHeight + translateY - coords[1];
             coords[3] = pageHeight + translateY - coords[3];
         }
-        // get the shading colorSpace
-        try
-        {
-            PDColorSpace cs = shadingType.getColorSpace();
-            if (!(cs instanceof PDDeviceRGB))
-            {
-                // we have to create an instance of the shading colorspace if it isn't RGB
-                shadingColorSpace = cs.getJavaColorSpace();
-                // get the tint transformation function if the colorspace has one
-                if (cs instanceof PDDeviceN)
-                {
-                    shadingTinttransform = ((PDDeviceN) cs).getTintTransform();
-                }
-                else if (cs instanceof PDSeparation)
-                {
-                    shadingTinttransform = ((PDSeparation) cs).getTintTransform();
-                }
-            }
-        }
-        catch (IOException exception)
-        {
-            LOG.error("error while creating colorSpace", exception);
-        }
-        // create the output colormodel using RGB+alpha as colorspace
-        ColorSpace outputCS = ColorSpace.getInstance(ColorSpace.CS_sRGB);
-        outputColorModel = new ComponentColorModel(outputCS, true, false, Transparency.TRANSLUCENT,
-                DataBuffer.TYPE_BYTE);
+        // transform the coords from user to device space
+        xform.transform(coords, 0, coords, 0, 2);
+
         // domain values
-        if (shadingType.getDomain() != null)
+        if (shading.getDomain() != null)
         {
-            domain = shadingType.getDomain().toFloatArray();
+            domain = shading.getDomain().toFloatArray();
         }
         else
         {
             // set default values
-            domain = new float[] { 0, 1 };
+            domain = new float[]
+            {
+                0, 1
+            };
         }
         // extend values
-        COSArray extendValues = shadingType.getExtend();
-        if (shadingType.getExtend() != null)
+        COSArray extendValues = shading.getExtend();
+        if (shading.getExtend() != null)
         {
             extend = new boolean[2];
             extend[0] = ((COSBoolean) extendValues.get(0)).getValue();
@@ -150,36 +122,83 @@ public class AxialShadingContext implements PaintContext
         else
         {
             // set default values
-            extend = new boolean[] { false, false };
+            extend = new boolean[]
+            {
+                false, false
+            };
         }
         // calculate some constants to be used in getRaster
         x1x0 = coords[2] - coords[0];
         y1y0 = coords[3] - coords[1];
         d1d0 = domain[1] - domain[0];
         denom = Math.pow(x1x0, 2) + Math.pow(y1y0, 2);
+        axialLength = Math.sqrt(denom);
 
         // get background values if available
-        COSArray bg = shadingType2.getBackground();
+        COSArray bg = shading.getBackground();
         if (bg != null)
         {
             background = bg.toFloatArray();
+            rgbBackground = convertToRGB(background);
         }
+        colorTable = calcColorTable();
     }
+
     /**
-     * {@inheritDoc}
+     * Calculate the color on the axial line and store them in an array.
+     *
+     * @return an array, index denotes the relative position, the corresponding
+     * value is the color on the axial line
      */
-    public void dispose() 
+    private int[] calcColorTable()
     {
-    	outputColorModel = null;
-        shadingColorSpace = null;
-        shadingTinttransform = null;
-        shadingType = null;
+        int[] map = new int[(int) axialLength + 1];
+        if (axialLength == 0 || d1d0 == 0)
+        {
+            try
+            {
+                float[] values = axialShadingType.evalFunction(domain[0]);
+                map[0] = convertToRGB(values);
+            }
+            catch (IOException exception)
+            {
+                LOG.error("error while processing a function", exception);
+            }
+        }
+        else
+        {
+            for (int i = 0; i <= axialLength; i++)
+            {
+                float t = domain[0] + d1d0 * i / (float) axialLength;
+                try
+                {
+                    float[] values = axialShadingType.evalFunction(t);
+                    map[i] = convertToRGB(values);
+                }
+                catch (IOException exception)
+                {
+                    LOG.error("error while processing a function", exception);
+                }
+            }
+        }
+        return map;
     }
 
     /**
      * {@inheritDoc}
      */
-    public ColorModel getColorModel() 
+    public void dispose()
+    {
+        outputColorModel = null;
+        shadingColorSpace = null;
+        shadingTinttransform = null;
+        axialShadingType = null;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public ColorModel getColorModel()
     {
         return outputColorModel;
     }
@@ -187,19 +206,36 @@ public class AxialShadingContext implements PaintContext
     /**
      * {@inheritDoc}
      */
-    public Raster getRaster(int x, int y, int w, int h) 
+    public Raster getRaster(int x, int y, int w, int h)
     {
         // create writable raster
         WritableRaster raster = getColorModel().createCompatibleWritableRaster(w, h);
-        boolean useBackground = false;
+        boolean useBackground;
         int[] data = new int[w * h * 4];
         for (int j = 0; j < h; j++)
         {
+            double currentY = y + j;
+            if (bboxRect != null)
+            {
+                if (currentY < minBBoxY || currentY > maxBBoxY)
+                {
+                    continue;
+                }
+            }
             for (int i = 0; i < w; i++)
             {
+                double currentX = x + i;
+                if (bboxRect != null)
+                {
+                    if (currentX < minBBoxX || currentX > maxBBoxX)
+                    {
+                        continue;
+                    }
+                }
+
                 useBackground = false;
-                double inputValue = x1x0 * (x + i - coords[0]);
-                inputValue += y1y0 * (y + j - coords[1]);
+                double inputValue = x1x0 * (currentX - coords[0]);
+                inputValue += y1y0 * (currentY - coords[1]);
                 // TODO this happens if start == end, see PDFBOX-1442
                 if (denom == 0)
                 {
@@ -217,12 +253,12 @@ public class AxialShadingContext implements PaintContext
                     inputValue /= denom;
                 }
                 // input value is out of range
-                if (inputValue < domain[0])
+                if (inputValue < 0)
                 {
                     // the shading has to be extended if extend[0] == true
                     if (extend[0])
                     {
-                        inputValue = domain[0];
+                        inputValue = 0;
                     }
                     else
                     {
@@ -237,12 +273,12 @@ public class AxialShadingContext implements PaintContext
                     }
                 }
                 // input value is out of range
-                else if (inputValue > domain[1])
+                else if (inputValue > 1)
                 {
                     // the shading has to be extended if extend[1] == true
                     if (extend[1])
                     {
-                        inputValue = domain[1];
+                        inputValue = 1;
                     }
                     else
                     {
@@ -256,44 +292,23 @@ public class AxialShadingContext implements PaintContext
                         }
                     }
                 }
-                float[] values = null;
-                int index = (j * w + i) * 4;
+                int value;
                 if (useBackground)
                 {
                     // use the given backgound color values
-                    values = background;
+                    value = rgbBackground;
                 }
                 else
                 {
-                    try
-                    {
-                        float input = (float) (domain[0] + (d1d0 * inputValue));
-                        values = shadingType.evalFunction(input);
-                    }
-                    catch (IOException exception)
-                    {
-                        LOG.error("error while processing a function", exception);
-                    }
+                    int key = (int) (inputValue * axialLength);
+                    value = colorTable[key];
                 }
-                // convert color values from shading colorspace to RGB if necessary
-                if (shadingColorSpace != null)
-                {
-                    if (shadingTinttransform != null)
-                    {
-                        try
-                        {
-                            values = shadingTinttransform.eval(values);
-                        }
-                        catch (IOException exception)
-                        {
-                            LOG.error("error while processing a function", exception);
-                        }
-                    }
-                    values = shadingColorSpace.toRGB(values);
-                }
-                data[index] = (int) (values[0] * 255);
-                data[index + 1] = (int) (values[1] * 255);
-                data[index + 2] = (int) (values[2] * 255);
+                int index = (j * w + i) * 4;
+                data[index] = value & 255;
+                value >>= 8;
+                data[index + 1] = value & 255;
+                value >>= 8;
+                data[index + 2] = value & 255;
                 data[index + 3] = 255;
             }
         }
@@ -303,42 +318,42 @@ public class AxialShadingContext implements PaintContext
 
     /**
      * Returns the coords values.
-     * 
+     *
      * @return the coords values as array
      */
-    public float[] getCoords() 
+    public float[] getCoords()
     {
         return coords;
     }
-        
+
     /**
      * Returns the domain values.
-     * 
+     *
      * @return the domain values as array
      */
-    public float[] getDomain() 
+    public float[] getDomain()
     {
         return domain;
     }
-        
+
     /**
      * Returns the extend values.
-     * 
+     *
      * @return the extend values as array
      */
-    public boolean[] getExtend() 
+    public boolean[] getExtend()
     {
         return extend;
     }
-    
-    /**
-     * Returns the function used for the shading tint transformation.
-     * 
-     * @return the shading tint transformation function
-     */
-    public PDFunction getShadingTintTransform() 
-    {
-        return shadingTinttransform;
-    }
 
+    /**
+     * Returns the function.
+     *
+     * @return the function
+     * @throws IOException if something goes wrong
+     */
+    public PDFunction getFunction() throws IOException
+    {
+        return axialShadingType.getFunction();
+    }
 }
