@@ -17,12 +17,9 @@
 package org.apache.pdfbox.pdmodel.graphics.shading;
 
 import java.awt.PaintContext;
-import java.awt.Transparency;
-import java.awt.color.ColorSpace;
+import java.awt.Rectangle;
 import java.awt.geom.AffineTransform;
 import java.awt.image.ColorModel;
-import java.awt.image.ComponentColorModel;
-import java.awt.image.DataBuffer;
 import java.awt.image.Raster;
 import java.awt.image.WritableRaster;
 import java.io.IOException;
@@ -32,129 +29,110 @@ import org.apache.commons.logging.LogFactory;
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBoolean;
 import org.apache.pdfbox.pdmodel.common.function.PDFunction;
-import org.apache.pdfbox.pdmodel.graphics.color.PDColorSpace;
-import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceN;
-import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceRGB;
-import org.apache.pdfbox.pdmodel.graphics.color.PDSeparation;
 import org.apache.pdfbox.util.Matrix;
 
 /**
- * This class represents the PaintContext of an radial shading.
- * 
- * @author lehmi
- * 
+ * AWT PaintContext for radial shading.
+ *
+ * Performance improvement done as part of GSoC2014, Tilman Hausherr is the
+ * mentor.
+ *
+ * @author Andreas Lehmkhler
+ * @author Shaola Ren
  */
-public class RadialShadingContext implements PaintContext 
+public class RadialShadingContext extends ShadingContext implements PaintContext
 {
-
-    private ColorModel outputColorModel;
-    private PDFunction function;
-    private ColorSpace shadingColorSpace;
-    private PDFunction shadingTinttransform;
-    private PDShadingType3 shadingType;
-
-    private float[] coords;
-    private float[] domain;
-    private boolean[] extend;
-    private float[] background;
-    private double x1x0; 
-    private double y1y0;
-    private double r1r0;
-    private double x1x0pow2;
-    private double y1y0pow2;
-    private double r0pow2;
-
-    private float d1d0;
-    private double denom;
-    
-    /**
-     * Log instance.
-     */
     private static final Log LOG = LogFactory.getLog(RadialShadingContext.class);
 
+    private PDShadingType3 radialShadingType;
+
+    private final float[] coords;
+    private final float[] domain;
+    private float[] background;
+    private int rgbBackground;
+    private final boolean[] extend;
+    private final double x1x0;
+    private final double y1y0;
+    private final double r1r0;
+    private final double x1x0pow2;
+    private final double y1y0pow2;
+    private final double r0pow2;
+    private final float d1d0;
+    private final double denom;
+
+    private final double longestDistance;
+    private final int[] colorTable;
+
     /**
      * Constructor creates an instance to be used for fill operations.
-     * 
-     * @param shadingType3 the shading type to be used
-     * @param colorModelValue the color model to be used
+     *
+     * @param shading the shading type to be used
+     * @param colorModel the color model to be used
      * @param xform transformation for user to device space
-     * @param ctm current transformation matrix
+     * @param ctm the transformation matrix
+     * @param dBounds device bounds
      * @param pageHeight height of the current page
-     * 
      */
-    public RadialShadingContext(PDShadingType3 shadingType3, ColorModel colorModelValue, 
-            AffineTransform xform, Matrix ctm, int pageHeight) 
+    public RadialShadingContext(PDShadingType3 shading, ColorModel colorModel, AffineTransform xform,
+            Matrix ctm, int pageHeight, Rectangle dBounds) throws IOException
     {
-        shadingType = shadingType3;
-        coords = shadingType3.getCoords().toFloatArray();
+        super(shading, colorModel, xform, ctm, pageHeight, dBounds);
+        this.radialShadingType = shading;
+        coords = shading.getCoords().toFloatArray();
+
         if (ctm != null)
         {
             // the shading is used in combination with the sh-operator
-            float[] coordsTemp = new float[coords.length]; 
             // transform the coords from shading to user space
-            ctm.createAffineTransform().transform(coords, 0, coordsTemp, 0, 1);
-            ctm.createAffineTransform().transform(coords, 3, coordsTemp, 3, 1);
+            ctm.createAffineTransform().transform(coords, 0, coords, 0, 1);
+            ctm.createAffineTransform().transform(coords, 3, coords, 3, 1);
             // scale radius to user space
             coords[2] *= ctm.getXScale();
             coords[5] *= ctm.getXScale();
+
             // move the 0,0-reference
-            coordsTemp[1] = pageHeight - coordsTemp[1];
-            coordsTemp[4] = pageHeight - coordsTemp[4];
-            // transform the coords from user to device space
-            xform.transform(coordsTemp, 0, coords, 0, 1);
-            xform.transform(coordsTemp, 3, coords, 3, 1);
-            // scale radius to device space
-            coords[2] *= xform.getScaleX();
-            coords[5] *= xform.getScaleX();
+            coords[1] = pageHeight - coords[1];
+            coords[4] = pageHeight - coords[4];
         }
         else
         {
             // the shading is used as pattern colorspace in combination
             // with a fill-, stroke- or showText-operator
-            float translateY = (float)xform.getTranslateY();
+            float translateY = (float) xform.getTranslateY();
             // move the 0,0-reference including the y-translation from user to device space
             coords[1] = pageHeight + translateY - coords[1];
             coords[4] = pageHeight + translateY - coords[4];
         }
-        // get the shading colorSpace
-        try
-        {
-            PDColorSpace cs = shadingType.getColorSpace();
-            if (!(cs instanceof PDDeviceRGB))
-            {
-                // we have to create an instance of the shading colorspace if it isn't RGB
-                shadingColorSpace = cs.getJavaColorSpace();
-                if (cs instanceof PDDeviceN)
-                {
-                    shadingTinttransform = ((PDDeviceN) cs).getTintTransform();
-                }
-                else if (cs instanceof PDSeparation)
-                {
-                    shadingTinttransform = ((PDSeparation) cs).getTintTransform();
-                }
-            }
-        }
-        catch (IOException exception)
-        {
-            LOG.error("error while creating colorSpace", exception);
-        }
-        // create the output colormodel using RGB+alpha as colorspace
-        ColorSpace outputCS = ColorSpace.getInstance(ColorSpace.CS_sRGB);
-        outputColorModel = new ComponentColorModel(outputCS, true, false, Transparency.TRANSLUCENT,
-                DataBuffer.TYPE_BYTE);
+
+        // transform the coords from user to device space
+        xform.transform(coords, 0, coords, 0, 1);
+        xform.transform(coords, 3, coords, 3, 1);
+
+        // scale radius to device space
+        coords[2] *= xform.getScaleX();
+        coords[5] *= xform.getScaleX();
+
+        // a radius is always positive
+        coords[2] = Math.abs(coords[2]);
+        coords[5] = Math.abs(coords[5]);
+
         // domain values
-        if (shadingType.getDomain() != null)
+        if (this.radialShadingType.getDomain() != null)
         {
-            domain = shadingType.getDomain().toFloatArray();
+            domain = shading.getDomain().toFloatArray();
         }
         else
         {
             // set default values
-            domain = new float[] { 0, 1 };
+            domain = new float[]
+            {
+                0, 1
+            };
         }
+
         // extend values
-        COSArray extendValues = shadingType.getExtend();
-        if (shadingType.getExtend() != null)
+        COSArray extendValues = shading.getExtend();
+        if (shading.getExtend() != null)
         {
             extend = new boolean[2];
             extend[0] = ((COSBoolean) extendValues.get(0)).getValue();
@@ -163,7 +141,10 @@ public class RadialShadingContext implements PaintContext
         else
         {
             // set default values
-            extend = new boolean[] { false, false };
+            extend = new boolean[]
+            {
+                false, false
+            };
         }
         // calculate some constants to be used in getRaster
         x1x0 = coords[3] - coords[0];
@@ -176,28 +157,97 @@ public class RadialShadingContext implements PaintContext
         d1d0 = domain[1] - domain[0];
 
         // get background values if available
-        COSArray bg = shadingType3.getBackground();
+        COSArray bg = shading.getBackground();
         if (bg != null)
         {
             background = bg.toFloatArray();
+            rgbBackground = convertToRGB(background);
+        }
+        longestDistance = getLongestDis();
+        colorTable = calcColorTable();
+    }
+
+    // get the longest distance of two points which are located on these two circles
+    private double getLongestDis()
+    {
+        double centerToCenter = Math.sqrt(x1x0pow2 + y1y0pow2);
+        double rmin, rmax;
+        if (coords[2] < coords[5])
+        {
+            rmin = coords[2];
+            rmax = coords[5];
+        }
+        else
+        {
+            rmin = coords[5];
+            rmax = coords[2];
+        }
+        if (centerToCenter + rmin <= rmax)
+        {
+            return 2 * rmax;
+        }
+        else
+        {
+            return rmin + centerToCenter + coords[5];
         }
     }
+
     /**
-     * {@inheritDoc}
+     * Calculate the color on the line connects two circles' centers and store
+     * the result in an array.
+     *
+     * @return an array, index denotes the relative position, the corresponding
+     * value the color
      */
-    public void dispose() 
+    private int[] calcColorTable()
     {
-    	outputColorModel = null;
-        function = null;
-        shadingColorSpace = null;
-        shadingTinttransform = null;
-        shadingType = null;
+        int[] map = new int[(int) longestDistance + 1];
+        if (longestDistance == 0 || d1d0 == 0)
+        {
+            try
+            {
+                float[] values = radialShadingType.evalFunction(domain[0]);
+                map[0] = convertToRGB(values);
+            }
+            catch (IOException exception)
+            {
+                LOG.error("error while processing a function", exception);
+            }
+        }
+        else
+        {
+            for (int i = 0; i <= longestDistance; i++)
+            {
+                float t = domain[0] + d1d0 * i / (float) longestDistance;
+                try
+                {
+                    float[] values = radialShadingType.evalFunction(t);
+                    map[i] = convertToRGB(values);
+                }
+                catch (IOException exception)
+                {
+                    LOG.error("error while processing a function", exception);
+                }
+            }
+        }
+        return map;
     }
 
     /**
      * {@inheritDoc}
      */
-    public ColorModel getColorModel() 
+    public void dispose()
+    {
+        outputColorModel = null;
+        radialShadingType = null;
+        shadingColorSpace = null;
+        shadingTinttransform = null;
+    }
+
+    /**
+     * {@inheritDoc}
+     */
+    public ColorModel getColorModel()
     {
         return outputColorModel;
     }
@@ -205,7 +255,7 @@ public class RadialShadingContext implements PaintContext
     /**
      * {@inheritDoc}
      */
-    public Raster getRaster(int x, int y, int w, int h) 
+    public Raster getRaster(int x, int y, int w, int h)
     {
         // create writable raster
         WritableRaster raster = getColorModel().createCompatibleWritableRaster(w, h);
@@ -214,8 +264,24 @@ public class RadialShadingContext implements PaintContext
         int[] data = new int[w * h * 4];
         for (int j = 0; j < h; j++)
         {
+            double currentY = y + j;
+            if (bboxRect != null)
+            {
+                if (currentY < minBBoxY || currentY > maxBBoxY)
+                {
+                    continue;
+                }
+            }
             for (int i = 0; i < w; i++)
             {
+                double currentX = x + i;
+                if (bboxRect != null)
+                {
+                    if (currentX < minBBoxX || currentX > maxBBoxX)
+                    {
+                        continue;
+                    }
+                }
                 useBackground = false;
                 float[] inputValues = calculateInputValues(x + i, y + j);
                 if (Float.isNaN(inputValues[0]) && Float.isNaN(inputValues[1]))
@@ -232,14 +298,14 @@ public class RadialShadingContext implements PaintContext
                 else
                 {
                     // choose 1 of the 2 values
-                    if (inputValues[0] >= domain[0] && inputValues[0] <= domain[1])
+                    if (inputValues[0] >= 0 && inputValues[0] <= 1)
                     {
-                        // both values are in the domain -> choose the larger one
-                        if (inputValues[1] >= domain[0] && inputValues[1] <= domain[1])
+                        // both values are in the range -> choose the larger one
+                        if (inputValues[1] >= 0 && inputValues[1] <= 1)
                         {
                             inputValue = Math.max(inputValues[0], inputValues[1]);
                         }
-                        // first value is in the domain, the second not -> choose first value
+                        // first value is in the range, the second not -> choose first value
                         else
                         {
                             inputValue = inputValues[0];
@@ -247,12 +313,13 @@ public class RadialShadingContext implements PaintContext
                     }
                     else
                     {
-                        // first value is not in the domain, but the second -> choose second value
-                        if (inputValues[1] >= domain[0] && inputValues[1] <= domain[1])
+                        // first value is not in the range, 
+                        // but the second -> choose second value
+                        if (inputValues[1] >= 0 && inputValues[1] <= 1)
                         {
                             inputValue = inputValues[1];
                         }
-                        // both are not in the domain
+                        // both are not in the range
                         else
                         {
                             if (extend[0] && extend[1])
@@ -278,12 +345,12 @@ public class RadialShadingContext implements PaintContext
                         }
                     }
                     // input value is out of range
-                    if (inputValue > domain[1])
+                    if (inputValue > 1)
                     {
-                        // the shading has to be extended if extend[1] == true
-                        if (extend[1])
+                        // extend shading if extend[1] is true and nonzero radius
+                        if (extend[1] && coords[5] > 0)
                         {
-                            inputValue = domain[1];
+                            inputValue = 1;
                         }
                         else
                         {
@@ -298,12 +365,12 @@ public class RadialShadingContext implements PaintContext
                         }
                     }
                     // input value is out of range
-                    else if (inputValue < domain[0])
+                    else if (inputValue < 0)
                     {
-                        // the shading has to be extended if extend[0] == true
-                        if (extend[0])
+                        // extend shading if extend[0] is true and nonzero radius
+                        if (extend[0] && coords[2] > 0)
                         {
-                            inputValue = domain[0];
+                            inputValue = 0;
                         }
                         else
                         {
@@ -318,44 +385,23 @@ public class RadialShadingContext implements PaintContext
                         }
                     }
                 }
-                float[] values = null;
-                int index = (j * w + i) * 4;
+                int value;
                 if (useBackground)
                 {
                     // use the given backgound color values
-                    values = background;
+                    value = rgbBackground;
                 }
                 else
                 {
-                    try
-                    {
-                        float input = (float) (domain[0] + (d1d0 * inputValue));
-                        values = shadingType.evalFunction(input);
-                    }
-                    catch (IOException exception)
-                    {
-                        LOG.error("error while processing a function", exception);
-                    }
+                    int key = (int) (inputValue * longestDistance);
+                    value = colorTable[key];
                 }
-                // convert color values from shading colorspace to RGB
-                if (shadingColorSpace != null)
-                {
-                    if (shadingTinttransform != null)
-                    {
-                        try
-                        {
-                            values = shadingTinttransform.eval(values);
-                        }
-                        catch (IOException exception)
-                        {
-                            LOG.error("error while processing a function", exception);
-                        }
-                    }
-                    values = shadingColorSpace.toRGB(values);
-                }
-                data[index] = (int) (values[0] * 255);
-                data[index + 1] = (int) (values[1] * 255);
-                data[index + 2] = (int) (values[2] * 255);
+                int index = (j * w + i) * 4;
+                data[index] = value & 255;
+                value >>= 8;
+                data[index + 1] = value & 255;
+                value >>= 8;
+                data[index + 2] = value & 255;
                 data[index + 3] = 255;
             }
         }
@@ -363,31 +409,24 @@ public class RadialShadingContext implements PaintContext
         return raster;
     }
 
-    private float[] calculateInputValues(int x, int y) 
+    private float[] calculateInputValues(int x, int y)
     {
-        
-        /** 
-         *  According to Adobes Technical Note #5600 we have to do the following 
-         *  
-         *  x0, y0, r0 defines the start circle
-         *  x1, y1, r1 defines the end circle
-         *  
-         *  The parametric equations for the center and radius of the gradient fill
-         *  circle moving between the start circle and the end circle as a function 
-         *  of s are as follows:
-         *  
-         *  xc(s) = x0 + s * (x1 - x0)
-         *  yc(s) = y0 + s * (y1 - y0)
-         *  r(s)  = r0 + s * (r1 - r0)
-         * 
-         *  Given a geometric coordinate position (x, y) in or along the gradient fill, 
-         *  the corresponding value of s can be determined by solving the quadratic 
-         *  constraint equation:
-         *  
-         *  [x - xc(s)]2 + [y - yc(s)]2 = [r(s)]2
-         *  
-         *  The following code calculates the 2 possible values of s
-         */
+        // According to Adobes Technical Note #5600 we have to do the following
+        //
+        // x0, y0, r0 defines the start circle x1, y1, r1 defines the end circle
+        //
+        // The parametric equations for the center and radius of the gradient fill circle moving
+        // between the start circle and the end circle as a function of s are as follows:
+        //
+        // xc(s) = x0 + s * (x1 - x0) yc(s) = y0 + s * (y1 - y0) r(s) = r0 + s * (r1 - r0)
+        //
+        // Given a geometric coordinate position (x, y) in or along the gradient fill, the
+        // corresponding value of s can be determined by solving the quadratic constraint equation:
+        //
+        // [x - xc(s)]2 + [y - yc(s)]2 = [r(s)]2
+        //
+        // The following code calculates the 2 possible values of s
+        //
         double p = -(x - coords[0]) * x1x0 - (y - coords[1]) * y1y0 - coords[2] * r1r0;
         double q = (Math.pow(x - coords[0], 2) + Math.pow(y - coords[1], 2) - r0pow2);
         double root = Math.sqrt(p * p - denom * q);
@@ -395,52 +434,58 @@ public class RadialShadingContext implements PaintContext
         float root2 = (float) ((-p - root) / denom);
         if (denom < 0)
         {
-            return new float[] { root1, root2 };
+            return new float[]
+            {
+                root1, root2
+            };
         }
         else
         {
-            return new float[] { root2, root1 };
+            return new float[]
+            {
+                root2, root1
+            };
         }
     }
-    
+
     /**
      * Returns the coords values.
-     * 
+     *
      * @return the coords values as array
      */
-    public float[] getCoords() 
+    public float[] getCoords()
     {
         return coords;
     }
-        
+
     /**
      * Returns the domain values.
-     * 
+     *
      * @return the domain values as array
      */
-    public float[] getDomain() 
+    public float[] getDomain()
     {
         return domain;
     }
-        
+
     /**
      * Returns the extend values.
-     * 
+     *
      * @return the extend values as array
      */
-    public boolean[] getExtend() 
+    public boolean[] getExtend()
     {
         return extend;
     }
-    
-    /**
-     * Returns the function used for the shading tint transformation.
-     * 
-     * @return the shading tint transformation function
-     */
-    public PDFunction getShadingTintTransform() 
-    {
-        return shadingTinttransform;
-    }
 
+    /**
+     * Returns the function.
+     *
+     * @return the function
+     * @throws IOException if something goes wrong
+     */
+    public PDFunction getFunction() throws IOException
+    {
+        return radialShadingType.getFunction();
+    }
 }
