@@ -16,11 +16,8 @@
  */
 package org.apache.pdfbox.pdmodel.graphics.xobject;
 
-import java.awt.AlphaComposite;
-import java.awt.Graphics2D;
+import java.awt.Color;
 import java.awt.Transparency;
-import java.awt.color.ColorSpace;
-import java.awt.image.ComponentColorModel;
 import java.awt.image.DataBuffer;
 import java.awt.image.DataBufferByte;
 import java.awt.image.BufferedImage;
@@ -29,6 +26,7 @@ import java.awt.image.IndexColorModel;
 import java.awt.image.WritableRaster;
 import java.io.IOException;
 import java.io.OutputStream;
+import javax.imageio.stream.MemoryCacheImageOutputStream;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -39,7 +37,6 @@ import org.apache.pdfbox.cos.COSName;
 import org.apache.pdfbox.pdmodel.PDDocument;
 import org.apache.pdfbox.pdmodel.common.PDStream;
 import org.apache.pdfbox.pdmodel.common.function.PDFunction;
-
 import org.apache.pdfbox.pdmodel.graphics.color.PDColorSpace;
 import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceGray;
 import org.apache.pdfbox.pdmodel.graphics.color.PDDeviceRGB;
@@ -79,7 +76,6 @@ public class PDPixelMap extends PDXObjectImage
     /**
      * Construct a pixel map image from an AWT image.
      * 
-     * 
      * @param doc The PDF document to embed the image in.
      * @param bi The image to read data from.
      *
@@ -88,70 +84,147 @@ public class PDPixelMap extends PDXObjectImage
     public PDPixelMap(PDDocument doc, BufferedImage bi) throws IOException
     {
         super( doc, PNG);
-        createImageStream(doc, bi);
+        createImageStream(doc, bi, false);
     }
 
-    private void createImageStream(PDDocument doc, BufferedImage bi) throws IOException
+    /**
+     * Construct a pixel map image from an AWT image.
+     * 
+     * @param doc The PDF document to embed the image in.
+     * @param bi The image to read data from.
+     * @param isMask true if this is a mask, false if not.
+     *
+     * @throws IOException If there is an error while embedding this image.
+     */
+    private PDPixelMap(PDDocument doc, BufferedImage bi, boolean isMask) throws IOException
+    {
+        super(doc, PNG);
+        createImageStream(doc, bi, isMask);
+    }
+
+    /**
+     * Create an image stream from an AWT image.
+     * 
+     * @param doc The PDF document to embed the image in.
+     * @param bi The image to read data from.
+     * @param isMask true if this is a mask, false if not. If true, the image
+     * stream will be forced to be DeviceGray.
+     * 
+     * @throws IOException If there is an error while embedding this image.
+     */
+    private void createImageStream(PDDocument doc, BufferedImage bi, boolean isMask) throws IOException
     {
         BufferedImage alphaImage = null;
-        BufferedImage rgbImage = null;
+        BufferedImage rgbImage;
         int width = bi.getWidth();
         int height = bi.getHeight();
         if (bi.getColorModel().hasAlpha())
         {
-            // extract the alpha information
-            WritableRaster alphaRaster = bi.getAlphaRaster();
-            ColorModel cm = new ComponentColorModel(ColorSpace.getInstance(ColorSpace.CS_GRAY), 
-                    false, false, Transparency.OPAQUE, DataBuffer.TYPE_BYTE);
-            alphaImage = new BufferedImage(cm, alphaRaster, false, null);
-            // create a RGB image without alpha
+            alphaImage = extractAlphaImage(bi);
+
+            // create RGB image without alpha
+            //BEWARE: the previous solution in the history 
+            // g.setComposite(AlphaComposite.Src) and g.drawImage()
+            // didn't work properly for TYPE_4BYTE_ABGR.
+            // alpha values of 0 result in a black dest pixel!!!
             rgbImage = new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR);
-            Graphics2D g = rgbImage.createGraphics();
-            g.setComposite(AlphaComposite.Src);
-            g.drawImage(bi, 0, 0, null);
+            for (int x = 0; x < width; ++x)
+            {
+                for (int y = 0; y < height; ++y)
+                {
+                    rgbImage.setRGB(x, y, bi.getRGB(x, y) & 0xFFFFFF);
+                }
+            }
         }
         else
         {
             rgbImage = bi;
         }
-        java.io.OutputStream os = null;
+        OutputStream os = null;
         try
         {
             int numberOfComponents = rgbImage.getColorModel().getNumComponents();
-            if (numberOfComponents == 3)
+            if (numberOfComponents != 3 && numberOfComponents != 1)
             {
-                setColorSpace( PDDeviceRGB.INSTANCE );
+                throw new IllegalStateException();
             }
-            else
+
+            int bpc;
+
+            // use FlateDecode compression
+            getPDStream().addCompression();
+            os = getCOSStream().createUnfilteredStream();
+
+            if (((isMask
+                    || bi.getType() == BufferedImage.TYPE_BYTE_GRAY)
+                    || bi.getType() == BufferedImage.TYPE_BYTE_BINARY)
+                    && bi.getColorModel().getPixelSize() <= 8)
             {
-                if (numberOfComponents == 1)
+                setColorSpace(new PDDeviceGray());
+                bpc = bi.getColorModel().getPixelSize();
+                if (bpc < 8)
                 {
-                    setColorSpace( new PDDeviceGray() );
+                    MemoryCacheImageOutputStream mcios = new MemoryCacheImageOutputStream(os);
+                    for (int y = 0; y < height; ++y)
+                    {
+                        for (int x = 0; x < width; ++x)
+                        {
+                            // grayscale images need one color per sample
+                            mcios.writeBits(bi.getRGB(x, y) & 0xFF, bpc);
+                        }
+                    }
+                    // padding
+                    while (mcios.getBitOffset() != 0)
+                    {
+                        mcios.writeBit(0);
+                    }
+                    mcios.flush();
+                    mcios.close();
                 }
                 else
                 {
-                    throw new IllegalStateException();
+                    //BEWARE: the gray values must be extracted from raster
+                    // and not from getRGB or the TYPE_BYTE_GRAY tests will fail
+                    DataBuffer dataBuffer = rgbImage.getData().getDataBuffer();
+                    for (int y = 0; y < height; ++y)
+                    {
+                        for (int x = 0; x < width; ++x)
+                        {
+                            // grayscale images need one color per sample
+                            os.write(dataBuffer.getElem(y * width + x));
+                        }
+                    }
                 }
             }
-            byte[] outData = new byte[width * height * numberOfComponents];
-            rgbImage.getData().getDataElements(0, 0, width, height, outData);
-            // add FlateDecode compression
-            getPDStream().addCompression();
-            os = getCOSStream().createUnfilteredStream();
-            os.write(outData);
-            
-            COSDictionary dic = getCOSStream();
-            dic.setItem( COSName.FILTER, COSName.FLATE_DECODE );
-            dic.setItem( COSName.SUBTYPE, COSName.IMAGE);
-            dic.setItem( COSName.TYPE, COSName.XOBJECT );
-            if(alphaImage != null)
+            else
             {
-                PDPixelMap smask = new PDPixelMap(doc, alphaImage);
+                // RGB
+                setColorSpace(PDDeviceRGB.INSTANCE);
+                bpc = 8;
+                for (int y = 0; y < height; ++y)
+                {
+                    for (int x = 0; x < width; ++x)
+                    {
+                        // rgb images need three colors per sample
+                        Color color = new Color(rgbImage.getRGB(x, y));
+                        os.write(color.getRed());
+                        os.write(color.getGreen());
+                        os.write(color.getBlue());
+                    }
+                }
+            }
+            COSDictionary dic = getCOSStream();
+            dic.setItem(COSName.FILTER, COSName.FLATE_DECODE);
+            dic.setItem(COSName.SUBTYPE, COSName.IMAGE);
+            dic.setItem(COSName.TYPE, COSName.XOBJECT);
+            if (alphaImage != null)
+            {
+                PDPixelMap smask = new PDPixelMap(doc, alphaImage, true);
                 dic.setItem(COSName.SMASK, smask);
             }
-            setBitsPerComponent( 8 );
-            setHeight( height );
-            setWidth( width );
+            setBitsPerComponent(bpc);
+            setHeight(height);
+            setWidth(width);
         }
         finally
         {
@@ -161,6 +234,7 @@ public class PDPixelMap extends PDXObjectImage
             }
         }
     }
+    
     /**
      * Returns a {@link java.awt.image.BufferedImage} of the COSStream
      * set in the constructor or null if the COSStream could not be encoded.
@@ -277,22 +351,7 @@ public class PDPixelMap extends PDXObjectImage
             }
             else
             {
-                if (colorspace instanceof PDICCBased)
-                {
-                    if (((PDICCBased)colorspace).getNumberOfComponents() == 1)
-                    {
-                        byte[] map = new byte[] {(byte)0xff};
-                        cm = new IndexColorModel(bpc, 1, map, map, map, Transparency.OPAQUE);
-                    }
-                    else
-                    {
-                        cm = colorspace.createColorModel( bpc );
-                    }
-                }
-                else
-                {
-                    cm = colorspace.createColorModel( bpc );
-                }
+                cm = colorspace.createColorModel( bpc );
             }
 
             LOG.debug("ColorModel: " + cm.toString());
@@ -400,4 +459,11 @@ public class PDPixelMap extends PDXObjectImage
         }
         return 1;
     }
+    
+    @Override
+    public void clear()
+    {
+        super.clear();
+        image = null;
+    }
 }
