@@ -19,7 +19,15 @@ package org.apache.pdfbox.pdmodel.graphics.xobject;
 import java.awt.AlphaComposite;
 import java.awt.Color;
 import java.awt.Graphics2D;
+import java.awt.Transparency;
+import java.awt.color.ColorSpace;
 import java.awt.image.BufferedImage;
+import java.awt.image.ColorModel;
+import java.awt.image.ComponentColorModel;
+import java.awt.image.DataBuffer;
+import java.awt.image.DataBufferByte;
+import java.awt.image.DataBufferInt;
+import java.awt.image.WritableRaster;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
@@ -187,9 +195,16 @@ public abstract class PDXObjectImage extends PDXObject
         }
         
         graphics.fillRect(0, 0, baseImage.getWidth(), baseImage.getHeight());
-        // assume default values ([0,1]) for the DecodeArray
-        // TODO DecodeArray == [1,0]
-        graphics.setComposite(AlphaComposite.DstIn);
+        COSArray decode = getDecode();
+        if (decode != null && decode.getInt(0) == 1)
+        {
+            // PDFBOX-1998
+            graphics.setComposite(AlphaComposite.DstOut);
+        }
+        else
+        {
+            graphics.setComposite(AlphaComposite.DstIn);
+        }
         graphics.drawImage(baseImage, null, 0, 0);
         graphics.dispose();
         return stencilMask;
@@ -473,4 +488,16 @@ public abstract class PDXObjectImage extends PDXObject
         return null;
     }
 
+    BufferedImage extractAlphaImage(BufferedImage bi)
+    {
+        WritableRaster alphaRaster = bi.getAlphaRaster();
+        BufferedImage alphaImage = new BufferedImage(alphaRaster.getWidth(), 
+                alphaRaster.getHeight(), 
+                bi.getTransparency() == Transparency.BITMASK ? 
+                BufferedImage.TYPE_BYTE_BINARY : 
+                BufferedImage.TYPE_BYTE_GRAY);
+        alphaImage.setData(alphaRaster);
+        return alphaImage;
+    }
+
 }
