@@ -120,7 +120,14 @@ public abstract class PDField implements COSObjectable
         String finalName = getPartialName();
         if (parentName != null)
         {
-            finalName = parentName + "." + finalName;
+            if (finalName != null)
+            {
+                finalName = parentName + "." + finalName;
+            }
+            else
+            {
+                finalName = parentName;
+            }
         }
         return finalName;
     }
@@ -528,7 +535,11 @@ public abstract class PDField implements COSObjectable
                 if (kidDictionary.getDictionaryObject(COSName.FT) != null
                         || (parent != null && parent.getDictionaryObject(COSName.FT) != null))
                 {
-                    kidsList.add(PDFieldFactory.createField(acroForm, kidDictionary));
+                    PDField field = PDFieldFactory.createField(acroForm, kidDictionary);
+                    if (field != null)
+                    {
+                        kidsList.add(field);
+                    }
                 }
                 else if ("Widget".equals(kidDictionary.getNameAsString(COSName.SUBTYPE)))
                 {
@@ -536,8 +547,11 @@ public abstract class PDField implements COSObjectable
                 }
                 else
                 {
-                    //
-                    kidsList.add(PDFieldFactory.createField(acroForm, kidDictionary));
+                    PDField field = PDFieldFactory.createField(acroForm, kidDictionary);
+                    if (field != null)
+                    {
+                        kidsList.add(field);
+                    }
                 }
             }
             retval = new COSArrayList<COSObjectable>(kidsList, kids);
