@@ -25,6 +25,8 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
 
 import org.apache.pdfbox.cos.COSArray;
 import org.apache.pdfbox.cos.COSBase;
@@ -41,6 +43,7 @@ import org.apache.pdfbox.pdmodel.PDDocumentCatalog;
 import org.apache.pdfbox.pdmodel.PDDocumentInformation;
 import org.apache.pdfbox.pdmodel.PDDocumentNameDictionary;
 import org.apache.pdfbox.pdmodel.PDPage;
+import org.apache.pdfbox.pdmodel.PDResources;
 import org.apache.pdfbox.pdmodel.common.COSArrayList;
 import org.apache.pdfbox.pdmodel.common.PDNumberTreeNode;
 import org.apache.pdfbox.pdmodel.common.PDStream;
@@ -62,9 +65,15 @@ import org.apache.pdfbox.pdmodel.interactive.form.PDFieldFactory;
  */
 public class PDFMergerUtility
 {
+    
+    /**
+     * Log instance.
+     */
+    private static final Log LOG = LogFactory.getLog(PDFMergerUtility.class);
+
     private static final String STRUCTURETYPE_DOCUMENT = "Document";
 
-    private List<InputStream> sources;
+    private final List<InputStream> sources;
     private String destinationFileName;
     private OutputStream destinationStream;
     private boolean ignoreAcroFormErrors = false;
@@ -209,21 +218,13 @@ public class PDFMergerUtility
         PDDocument source;
         if (sources != null && sources.size() > 0)
         {
-            java.util.Vector<PDDocument> tobeclosed = new java.util.Vector<PDDocument>();
+            ArrayList<PDDocument> tobeclosed = new ArrayList<PDDocument>();
 
             try
             {
                 Iterator<InputStream> sit = sources.iterator();
-                sourceFile = sit.next();
-                if (isNonSeq)
-                {
-                    destination = PDDocument.loadNonSeq(sourceFile, scratchFile);
-                }
-                else
-                {
-                    destination = PDDocument.load(sourceFile);
-                }
-
+                destination = new PDDocument();
+                
                 while (sit.hasNext())
                 {
                     sourceFile = sit.next();
@@ -303,23 +304,6 @@ public class PDFMergerUtility
             destCatalog.setOpenAction(srcCatalog.getOpenAction());
         }
 
-        // maybe there are some shared resources for all pages
-        COSDictionary srcPages = (COSDictionary) srcCatalog.getCOSDictionary().getDictionaryObject(COSName.PAGES);
-        COSDictionary srcResources = (COSDictionary) srcPages.getDictionaryObject(COSName.RESOURCES);
-        COSDictionary destPages = (COSDictionary) destCatalog.getCOSDictionary().getDictionaryObject(COSName.PAGES);
-        COSDictionary destResources = (COSDictionary) destPages.getDictionaryObject(COSName.RESOURCES);
-        if (srcResources != null)
-        {
-            if (destResources != null)
-            {
-                destResources.mergeInto(srcResources);
-            }
-            else
-            {
-                destPages.setItem(COSName.RESOURCES, srcResources);
-            }
-        }
-
         PDFCloneUtility cloner = new PDFCloneUtility(destination);
 
         try
@@ -339,14 +323,19 @@ public class PDFMergerUtility
                 }
             }
         }
-        catch (Exception e)
+        catch (IOException e)
         {
             // if we are not ignoring exceptions, we'll re-throw this
             if (!ignoreAcroFormErrors)
             {
-                throw (IOException) e;
+                LOG.error (e, e);
+                throw new IOException(e.getMessage());
             }
         }
+        catch (Exception e)
+        {
+            throw new RuntimeException(e);
+        }
 
         COSArray destThreads = (COSArray) destCatalog.getCOSDictionary().getDictionaryObject(COSName.THREADS);
         COSArray srcThreads = (COSArray) cloner.cloneForNewDocument(destCatalog.getCOSDictionary().getDictionaryObject(
@@ -372,7 +361,6 @@ public class PDFMergerUtility
             {
                 cloner.cloneMerge(srcNames, destNames);
             }
-
         }
 
         PDDocumentOutline destOutline = destCatalog.getDocumentOutline();
@@ -409,7 +397,7 @@ public class PDFMergerUtility
         if (srcLabels != null)
         {
             int destPageCount = destination.getNumberOfPages();
-            COSArray destNums = null;
+            COSArray destNums;
             if (destLabels == null)
             {
                 destLabels = new COSDictionary();
@@ -498,7 +486,6 @@ public class PDFMergerUtility
             }
         }
 
-        // finally append the pages
         List<PDPage> pages = srcCatalog.getAllPages();
         Iterator<PDPage> pageIter = pages.iterator();
         HashMap<COSDictionary, COSDictionary> objMapping = new HashMap<COSDictionary, COSDictionary>();
@@ -509,6 +496,8 @@ public class PDFMergerUtility
             newPage.setCropBox(page.findCropBox());
             newPage.setMediaBox(page.findMediaBox());
             newPage.setRotation(page.findRotation());
+            // this is smart enough to just create references for resources that are used on multiple pages
+            newPage.setResources(new PDResources((COSDictionary) cloner.cloneForNewDocument(page.findResources())));                        
             if (mergeStructTree)
             {
                 updateStructParentEntries(newPage, destParentTreeNextKey);
