@@ -96,6 +96,12 @@ public class TestDateUtil extends TestCase
         }   
     }
     
+    public void testEmptyDate() throws Exception 
+    {
+       	assertNull(DateConverter.toCalendar(""));
+    	assertNull(DateConverter.toCalendar(new COSString("")));
+    }
+    
     /**
      * Calendar.equals test case.
      * 
